#  tests for lue-0.2.0-py37h1d2e621_11 (this is a generated file);
print('===== testing package: lue-0.2.0-py37h1d2e621_11 =====');
print('running run_test.py');
#  --- run_test.py (begin) ---
import lue
import lue.data_model as ldm

print('lue version: {}'.format(lue.lue_version))
print('git hash:    {}'.format(lue.git_short_sha1))

res = ldm.SpaceDomain
assert res is not None
#  --- run_test.py (end) ---

print('===== lue-0.2.0-py37h1d2e621_11 OK =====');
